// *************************************************************************
// Copyright (c) 1998 SCIDYNE
// File				:	cp_ex1.c
// By					:	Mark W. Durgin
// Date				:	12-18-98
// Revision		:	1.00
// IOCP-74 Demonstration program
// *************************************************************************
//	This program demonstrates interfacing the IOCP-74 to a host
// 	running Borland turbo C++ version 5.0 or equivilent. This program
//	assumes that the IOCP-74 has been programmed and is running the
//	CP_EX1.ASM (PIC assembly langauge) demonstration program.
//
// 	Note: This program was tested on a 25mhz 386SX embedded PC/104
//				host.  Some timing changes maybe required to run on
//				higher performance computers.
//
// *************************************************************************

// --- Program includes
#include <stdio.h>
#include <conio.h>

// --- Define program constants
#define	TRUE	1								// Logic TRUE state
#define FALSE	!TRUE						// Logic FALSE state
#define	BASE_ADDR 0x300				// Define where in IO map IOCP-74 is located
#define	DATA_REG  BASE_ADDR+0 // Data Array Register location
#define	CMD_REG   BASE_ADDR+1	// Data Array Register location
#define	BRD_RESET BASE_ADDR+2	// Board Reset
#define	AUTO_INC_MASK 32 			// Bit pattern for auto-inc enable, bit 5 set
#define ARRAY_SIZE	 16				// Size of read and write arrays
#define	DELAY_MODULUS	125 		// Determines delay between IOCP-74 accesses

// --- Define structures used in this program
struct struct_read_array { 	// Interpetation of "bytes" read from IOCP-74
	unsigned int		ain0;			// (0,1)	Analog input #0 (12-bit)
	unsigned int		ain1;			// (2,3)	Analog input #1 (12-bit)
  unsigned char		ain2;			// (4)  	Analog input #2 (8-bit)
  unsigned char		ain3;			// (5)  	Analog input #3 (8-bit)
  unsigned char		ain4;			// (6)  	Analog input #4 (8-bit)
	};

struct struct_write_array {	// Interpetation of "bytes" written to IOCP-74
	unsigned int		aout0; 		// (0,1) Analog input #0 (12-bit)
  unsigned int		aout1;	 	// (2,3) Analog input #1 (12-bit)
  };

// --- Declare global variables
unsigned char	read_array[ARRAY_SIZE];			// Holds incoming data read FROM IOCP-74
unsigned char	write_array[ARRAY_SIZE];		// Holds outgoing data written TO IOCP-74

// --- Declare function prototypes
void interaccess_delay(void);  						// Performs a delay between IOCP-74 accesses

// --- Main program
void main(void)
{
   // --- Define local variables
   struct struct_read_array  *rd_pntr; 	// Create read array structure overlay pointer
   struct struct_write_array *wr_pntr; 	// Create write array structure overlay pointer
   unsigned int	dly,i; 									// Used for general purposes

   // --- Initz the program
   rd_pntr = (struct struct_read_array *) &read_array[0]; 		// Initz the read structure pointer
   wr_pntr = (struct struct_write_array *) 	&write_array[0]; 	// Initz the write structure pointer

   clrscr();  // Clear the display and show a banner
   printf ("----------------------------\n");
   printf ("         SCIDYNE\n");
   printf ("IOCP-74 Evaluation Program\n");
   printf ("     CP_EX1.C     V1.00\n");
   printf ("----------------------------\n");

   /*** --- Reset the IOCP
   NOTE: This isn't actually required because a host power-on-reset
   also generates an IOCP-74 reset. However, it is a good idea to
   include it during software development so that the IOCP-74 starts
   in a known state without having to reset the host.
   ***/

   outp(BRD_RESET, 0xff);  	// Generate a IOCP-74 hardware reset
   printf ("Hardware being reset ");
   for(i=0; i <= 11; i++)		// Wait for reset to take
   {
   		printf(".");
      for(dly=0; dly < 32000; dly++);
   }
   printf(" Done\n\n");

   // --- This is the main loop
   while(TRUE)
   {
   		printf ("-----------------------------------\n");

   		// --- Prompt user for analog output values
   		printf ("--- Analog Outputs ---\n");

   		printf ("AO-CH0: ");
   		scanf("%4d",&wr_pntr->aout0);  	// Input value for channel-0
   		printf ("AO-CH1: ");
   		scanf("%4d",&wr_pntr->aout1);  	// Input value for channel-1
   		printf("\n");


   		// --- Output the write structure data to the IOCP-74
   		outp(CMD_REG, 0 + AUTO_INC_MASK);  	// Set IOCP data pointer to 1st location, enable auto inc
   		interaccess_delay();  							// Allow time for the PIC to respond

   		for (i=0; i < sizeof(struct_write_array); i++)
   		{
    		outp(DATA_REG, write_array[i]);  	// Write one data byte, HOST ---> IOCP-74
      	interaccess_delay();              // Allow time for the PIC to respond
   		}


   		// --- Load the read array from IOCP data
   		outp(CMD_REG, 0 + AUTO_INC_MASK); 	// Set IOCP data pointer to 1st location, enable auto inc
   		interaccess_delay();  							// Allow time for the PIC to respond

   		for (i=0; i < sizeof(struct_read_array); i++)
   		{
      	read_array[i] = inp(DATA_REG);  	// Read one data byte, HOST <--- IOCP-74
      	interaccess_delay();              // Allow time for the PIC to respond
   		}

   		// --- Display the analog input data using overlay structure
   		printf("AIN0: %4d \n",rd_pntr->ain0);
   		printf("AIN1: %4d \n",rd_pntr->ain1);
   		printf("AIN2: %4d \n",rd_pntr->ain2);
   		printf("AIN3: %4d \n",rd_pntr->ain3);
   		printf("AIN4: %4d \n",rd_pntr->ain4);

   }
}

// --- This function is a software time delay.
//     NOTE:  Adjust DELAY_MODULUS constant for your host plateform
void interaccess_delay(void)
{
	unsigned int j;

	for (j=0; j < DELAY_MODULUS; j++);
  return;
}



